/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  Based on egg-toggle-action from libegg , which does not have copyright header.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-toggle-action.h"
#include "eggtoggletoolbutton.h"
#include "gul-gobject-misc.h"


static void		galeon_toggle_action_init		(GaleonToggleAction *action);
static void		galeon_toggle_action_class_init		(GaleonToggleActionClass *class);
static void		galeon_toggle_action_activate		(EggAction *action);
static void		galeon_toggle_action_real_toggled	(GaleonToggleAction *action);
static void		galeon_toggle_action_connect_proxy	(EggAction *action,
								 GtkWidget *proxy);
static void		galeon_toggle_action_disconnect_proxy	(EggAction *action,
								 GtkWidget *proxy);

static GObjectClass *parent_class = NULL;

enum
{
	GALEON_TOGGLE_ACTION_TOGGLED,
	GALEON_TOGGLE_ACTION_LAST_SIGNAL
} GaleonToggleActionSignalsEnum;
static guint GaleonToggleActionSignals[GALEON_TOGGLE_ACTION_LAST_SIGNAL];


MAKE_GET_TYPE (galeon_toggle_action, "GaleonToggleAction", GaleonToggleAction, galeon_toggle_action_class_init, 
	       galeon_toggle_action_init, GALEON_TYPE_ACTION);

static void
galeon_toggle_action_class_init (GaleonToggleActionClass *class)
{
	EggActionClass *action_class;
	
	parent_class = g_type_class_peek_parent (class);
	action_class = EGG_ACTION_CLASS (class);
	
	action_class->activate = galeon_toggle_action_activate;
	action_class->connect_proxy = galeon_toggle_action_connect_proxy;
	action_class->disconnect_proxy = galeon_toggle_action_disconnect_proxy;
	
	action_class->menu_item_type = GTK_TYPE_CHECK_MENU_ITEM;
	action_class->toolbar_item_type = EGG_TYPE_TOGGLE_TOOL_BUTTON;
	
	class->toggled = galeon_toggle_action_real_toggled;
	
	GaleonToggleActionSignals[GALEON_TOGGLE_ACTION_TOGGLED] =
		g_signal_new ("toggled",
			      G_OBJECT_CLASS_TYPE (class),
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GaleonToggleActionClass, toggled),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
}

static void
galeon_toggle_action_init (GaleonToggleAction *action)
{
	action->active = FALSE;
}

static void
galeon_toggle_action_activate (EggAction *action)
{
	GaleonToggleAction *toggle_action;
	
	g_return_if_fail (GALEON_IS_TOGGLE_ACTION (action));
	
	toggle_action = GALEON_TOGGLE_ACTION (action);
	
	toggle_action->active = !toggle_action->active;
	
	galeon_toggle_action_toggled (toggle_action);
}

static void
galeon_toggle_action_synch_proxy (GaleonToggleAction *action, GtkWidget *proxy)
{
	if (GTK_IS_CHECK_MENU_ITEM (proxy))
	{
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (proxy), action->active);
	}
	else if (EGG_IS_TOGGLE_TOOL_BUTTON (proxy))
	{
		egg_toggle_tool_button_set_active (EGG_TOGGLE_TOOL_BUTTON (proxy), action->active);
	}
	else
	{
		g_warning ("Don't know how to toggle `%s' widgets",
			   G_OBJECT_TYPE_NAME (proxy));
	}
}

static void
galeon_toggle_action_real_toggled (GaleonToggleAction *action)
{
	GSList *slist;
	
	g_return_if_fail (GALEON_IS_TOGGLE_ACTION (action));
	
	for (slist = EGG_ACTION (action)->proxies; slist; slist = slist->next)
	{
		GtkWidget *proxy = slist->data;
		
		egg_action_block_activate_from (EGG_ACTION (action), proxy);

		galeon_toggle_action_synch_proxy (action, proxy);
		
		egg_action_unblock_activate_from (EGG_ACTION (action), proxy);
	}
}

static void
galeon_toggle_action_connect_proxy (EggAction *action, GtkWidget *proxy)
{
	GaleonToggleAction *toggle_action;
	
	toggle_action = GALEON_TOGGLE_ACTION (action);
	
	/* do this before hand, so that we don't call the "activate" handler */
	galeon_toggle_action_synch_proxy (toggle_action, proxy);
	
	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
}

static void
galeon_toggle_action_disconnect_proxy (EggAction *action, GtkWidget *proxy)
{
	(* EGG_ACTION_CLASS (parent_class)->disconnect_proxy) (action, proxy);
}

/**
 * galeon_toggle_action_toggled:
 * @action: the action object
 *
 * Emits the "toggled" signal on the toggle action.
 */
void
galeon_toggle_action_toggled (GaleonToggleAction *action)
{
	g_return_if_fail (GALEON_IS_TOGGLE_ACTION (action));
	
	g_signal_emit (action, GaleonToggleActionSignals[GALEON_TOGGLE_ACTION_TOGGLED], 0);
}

/**
 * galeon_toggle_action_set_active:
 * @action: the action object
 * @is_active: whether the action should be checked or not
 *
 * Sets the checked state on the toggle action.
 */
void
galeon_toggle_action_set_active (GaleonToggleAction *action, gboolean is_active)
{
	g_return_if_fail (GALEON_IS_TOGGLE_ACTION (action));
	
	is_active = !!is_active;
	
	if (action->active != is_active)
	{
		egg_action_activate (EGG_ACTION (action));
	}
}

/**
 * galeon_toggle_action_get_active:
 * @action: the action object
 *
 * Returns: the checked state of the toggle action
 */
gboolean
galeon_toggle_action_get_active (GaleonToggleAction *action)
{
	g_return_val_if_fail (GALEON_IS_TOGGLE_ACTION (action), FALSE);
	
	return action->active;
}
